<?php
/*
	DREAMWEAVER TEMPLATE PARSER
	
	This class will read a Dreamweaver template file, and break it into "sections."
	The sections will be placed into an associative array.
	
	This was written in order for me to use Dreamwever templates in CMS systems
	like WordPress and Gallery. This allows me to have a blank template file sitting
	in an appropriate place, and I can grab it and parse it in order to shoehorn the
	CMS data into it, which gives me the full advantage of the Dreamweaver template
	system.
	
	It can be used either by accessing the template in "chunks," or by loading the
	template up with data, then displaying the entire file.
	
	KNOWN ISSUE: Don't declare multiple editable regions on the same line.
	
	Version: 1.0.1 (Added that caveat above)
	Date Mod: September 21st, 2005
	Author: Chris Marshall (cmarshall@xemaps.com)
	
	Version: 1.0
	Date Mod: September 20th, 2005
	Author: Chris Marshall (cmarshall@xemaps.com)
	
	License:
		Take me, You Gypsy Stallion.
		Open Source.
		No guarantees, no claims of ownership, but don't try to copyright this.
		
	Origination: http://www.cmarshall.net/DreamweaverTemplate
*/

class DreamweaverTemplate
{
	/*
	* ****************************************
	*                 Members
	* ****************************************
	*/
	
	/**
	* This contains a POSIX path to the file to be parsed.
	*
	* @var object DreamweaverTemplate $_file_to_parse
	* @access private
	*/
	var $_file_to_parse;

	/**
	* This contains the parsed file as an associative array.
	*
	* @var object DreamweaverTemplate $_parsed_file
	* @access private
	*/
	var $_parsed_file;

	/*
	* ****************************************
	*                 Methods
	* ****************************************
	*/
	
	/**
	* Constructor
	* @param string a POSIX path to the file to be parsed
	* @access public
	*/
	function DreamweaverTemplate ( $in_file_to_parse )
		{
		assert ( '!empty($in_file_to_parse)' );
		$this->_file_to_parse = $in_file_to_parse;
		$this->_ParseFile();
		}
	
	/**
	* Parse the file
	* @access private
	*/
	function _ParseFile()
		{
		$this->_parsed_file = "";	// Make sure we are empty.
		// Read in the file
		$template_file = file_get_contents ( $this->_file_to_parse );
		$template_file = preg_split ( '/<!-- .*?BeginEditable [^"]*"|<!-- .*?EndEditable -->/',  $template_file );
		if ( count ( $template_file ) )
			{
			$index = 0;
			foreach ( $template_file as $section )
				{
				$key = "";
				$value = "";
				list ( $key, $value ) = preg_split ( '/-->/', $section );

				if ( !preg_match ( '/^[^"]*?" /', $key ) )
					{
					$key = $section;
					$value = "";
					}
				
				if ( !$value && !preg_match ( '/^[^"]*?" /', $key ) )
					{
					$value = $key;
					$key = "locked_$index";
					$index++;
					}
				else
					{
					$key = str_replace ( '" ', '', $key );
					$key = "editable_$key";
					}
				
				$this->_parsed_file[$key] = $value;
				}
			}
		}
	
	/**
	* Return the file from the end of one editable region to another
	* All data in all editable regions will be returned, with the exception of the destination region.
	* That data will only be returned if the $in_include_data is true.
	* $in_from and $in_to must be editable regions.
	* The "locked_" and "editable_" prefixes are not to be passed in.
	* This may seem counter-intuitive, as you send in the destination before the source, but this makes
	* the function easier to use.
	* @param string the editable region you want to stop at (blank is the end of the file)
	* @param string the editable region that you wish to start at (blank is the beginning of the file)
	* @param bool nonzero if you want to include the data already in the destination editable region
	* @access public
	*/
	function GetFileData ( $in_to = "", $in_from = "", $in_include_data = false )
		{
		$ret = "";
		$start_reading = false;
		
		if ( !$in_from )
			{
			$start_reading = true;
			}
		else
			{
			$in_from = "editable_$in_from";
			}
		
		if ( $in_to )
			{
			$in_to = "editable_$in_to";
			}
		
		if ( count ( $this->_parsed_file ) )
			{
			reset ( $this->_parsed_file );
			while ( list ( $key, $value ) = each ( $this->_parsed_file ) )
				{
				if ( !$start_reading && ($key == $in_from) )
					{
					$start_reading = true;
					continue;
					}
				
				if ( $start_reading )
					{
					if ( ($key == $in_to) && !$in_include_data )
						{
						break;
						}
					
					$ret .= $value;

					if ( $key == $in_to )
						{
						break;
						}
					}
				}
			}
		
		return $ret;
		}
	
	/**
	* Returns an array with the editable region names (without the "editable_" prefix)
	* @access public
	*/
	function GetEditableRegions()
		{
		$ret = array();
		if ( count ( $this->_parsed_file ) )
			{
			reset ( $this->_parsed_file );
			while ( list ( $key, $value ) = each ( $this->_parsed_file ) )
				{
				if ( preg_match ( "/editable_/", $key ) )
					{
					array_push ( $ret, preg_replace ( "/editable_/", "", $key ) );
					}
				}
			}
		
		return $ret;
		}
	
	/**
	* Returns a copy of the actual array
	* @access public
	*/
	function GetParsedArray()
		{
		return $this->_parsed_file;
		}
	
	/**
	* Returns the data within an editable region
	* @param string the editable region you want to see (must be valid, and does not include the "editable_" prefix)
	* @access public
	*/
	function GetEditableData ( $in_editable_region )
		{
		assert ( '!empty($in_editable_region)' );
		return $this->_parsed_file["editable_$in_editable_region"];
		}
	
	/**
	* Sets data into an editable region
	* Remember that, when accessing this, you need to specify the $in_include_data flag.
	* @param string the editable region you want to edit (must be valid, and does not include the "editable_" prefix)
	* @param string the data you want to place in that region (can be empty)
	* @access public
	*/
	function SetEditableData ( $in_editable_region, $in_data = "" )
		{
		assert ( '!empty($in_editable_region)' );
		
		$this->_parsed_file["editable_$in_editable_region"] = $in_data;
		}
};
?>